@@ -51,19 +51,48 @@ static bool panel_utils_grab_available()
 {
 	bool grab_succeed = false;
 
-	// Don't try to get the grab for longer then 1/4 second
 	GdkWindow* root = gdk_screen_get_root_window(xfce_gdk_screen_get_active(NULL));
+	GdkDisplay* display = gdk_display_get_default();
+#if GTK_CHECK_VERSION(3,20,0)
+	GdkSeat* seat = gdk_display_get_default_seat(display);
+
+	// Don't try to get the grab for longer then 1/4 second
+	for (guint i = 0; i < (G_USEC_PER_SEC / 400); ++i)
+	{
+		if (gdk_seat_grab(seat, root, GDK_SEAT_CAPABILITY_ALL, true, NULL, NULL, NULL, NULL))
+		{
+			gdk_seat_ungrab(seat);
+			grab_succeed = true;
+			break;
+		}
+		g_usleep(100);
+	}
+#else
+	GdkDeviceManager* device_manager = gdk_display_get_device_manager(display);
+	GdkDevice* pointer = gdk_device_manager_get_client_pointer(device_manager);
+	GdkDevice* keyboard = gdk_device_get_associated_device(pointer);
+
+	// Don't try to get the grab for longer then 1/4 second
 	GdkGrabStatus grab_pointer = GDK_GRAB_FROZEN;
 	GdkGrabStatus grab_keyboard = GDK_GRAB_FROZEN;
-	for (guint i = 0; i < (G_USEC_PER_SEC / 4 / 100); ++i)
+	for (guint i = 0; i < (G_USEC_PER_SEC / 400); ++i)
 	{
-		grab_keyboard = gdk_keyboard_grab(root, true, GDK_CURRENT_TIME);
+		grab_keyboard = gdk_device_grab(keyboard,
+				root,
+				GDK_OWNERSHIP_NONE,
+				true,
+				GDK_ALL_EVENTS_MASK,
+				NULL,
+				GDK_CURRENT_TIME);
 		if (grab_keyboard == GDK_GRAB_SUCCESS)
 		{
-			const GdkEventMask pointer_mask = static_cast<GdkEventMask>(GDK_BUTTON_PRESS_MASK
-					| GDK_BUTTON_RELEASE_MASK | GDK_ENTER_NOTIFY_MASK
-					| GDK_LEAVE_NOTIFY_MASK | GDK_POINTER_MOTION_MASK);
-			grab_pointer = gdk_pointer_grab(root, true, pointer_mask, NULL, NULL, GDK_CURRENT_TIME);
+			grab_pointer = gdk_device_grab(pointer,
+					root,
+					GDK_OWNERSHIP_NONE,
+					true,
+					GDK_ALL_EVENTS_MASK,
+					NULL,
+					GDK_CURRENT_TIME);
 			if (grab_pointer == GDK_GRAB_SUCCESS)
 			{
 				grab_succeed = true;
@@ -76,12 +105,13 @@ static bool panel_utils_grab_available()
 	// Release the grab so the menu window can take it
 	if (grab_pointer == GDK_GRAB_SUCCESS)
 	{
-		gdk_pointer_ungrab(GDK_CURRENT_TIME);
+		gdk_device_ungrab(pointer, GDK_CURRENT_TIME);
 	}
 	if (grab_keyboard == GDK_GRAB_SUCCESS)
 	{
-		gdk_keyboard_ungrab(GDK_CURRENT_TIME);
+		gdk_device_ungrab(keyboard, GDK_CURRENT_TIME);
 	}
+#endif
 
 	if (!grab_succeed)
 	{
@@ -122,11 +152,15 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	m_button = xfce_panel_create_toggle_button();
 	gtk_widget_set_name(m_button, "whiskermenu-button");
 	gtk_button_set_relief(GTK_BUTTON(m_button), GTK_RELIEF_NONE);
+#if GTK_CHECK_VERSION(3,20,0)
+	gtk_widget_set_focus_on_click(GTK_WIDGET(m_button), false);
+#else
 	gtk_button_set_focus_on_click(GTK_BUTTON(m_button), false);
+#endif
 	g_signal_connect_slot(m_button, "toggled", &Plugin::button_toggled, this);
 	gtk_widget_show(m_button);
 
-	m_button_box = GTK_BOX(gtk_hbox_new(false, 2));
+	m_button_box = GTK_BOX(gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2));
 	gtk_container_add(GTK_CONTAINER(m_button), GTK_WIDGET(m_button_box));
 	gtk_container_set_border_width(GTK_CONTAINER(m_button_box), 0);
 	gtk_widget_show(GTK_WIDGET(m_button_box));
@@ -153,11 +187,7 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	// Connect plugin signals to functions
 	g_signal_connect(plugin, "free-data", G_CALLBACK(whiskermenu_free), this);
 	g_signal_connect_slot<XfcePanelPlugin*>(plugin, "configure-plugin", &Plugin::configure, this);
-#if (LIBXFCE4PANEL_CHECK_VERSION(4,9,0))
 	g_signal_connect_slot(plugin, "mode-changed", &Plugin::mode_changed, this);
-#else
-	g_signal_connect_slot(plugin, "orientation-changed", &Plugin::orientation_changed, this);
-#endif
 	g_signal_connect_slot(plugin, "remote-event", &Plugin::remote_event, this);
 	g_signal_connect_slot<XfcePanelPlugin*>(plugin, "save", &Plugin::save, this);
 	g_signal_connect_slot<XfcePanelPlugin*>(plugin, "about", &Plugin::show_about, this);
@@ -167,11 +197,7 @@ Plugin::Plugin(XfcePanelPlugin* plugin) :
 	xfce_panel_plugin_menu_show_configure(plugin);
 	xfce_panel_plugin_menu_insert_item(plugin, GTK_MENU_ITEM(wm_settings->command[Settings::CommandMenuEditor]->get_menuitem()));
 
-#if (LIBXFCE4PANEL_CHECK_VERSION(4,9,0))
 	mode_changed(m_plugin, xfce_panel_plugin_get_mode(m_plugin));
-#else
-	orientation_changed(m_plugin, xfce_panel_plugin_get_orientation(m_plugin));
-#endif
 
 	g_signal_connect_slot<GtkWidget*,GtkStyle*>(m_button, "style-set", &Plugin::update_size, this);
 	g_signal_connect_slot<GtkWidget*,GdkScreen*>(m_button, "screen-changed", &Plugin::update_size, this);
@@ -323,24 +349,16 @@ void Plugin::menu_hidden()
 void Plugin::configure()
 {
 	ConfigurationDialog* dialog = new ConfigurationDialog(this);
-	g_signal_connect_slot<GtkObject*>(dialog->get_widget(), "destroy", &Plugin::save, this);
+	g_signal_connect_slot<GtkWidget*>(dialog->get_widget(), "destroy", &Plugin::save, this);
 }
 
 //-----------------------------------------------------------------------------
 
-#if (LIBXFCE4PANEL_CHECK_VERSION(4,9,0))
 void Plugin::mode_changed(XfcePanelPlugin*, XfcePanelPluginMode mode)
 {
 	gtk_label_set_angle(m_button_label, (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ? 270: 0);
 	update_size();
 }
-#else
-void Plugin::orientation_changed(XfcePanelPlugin*, GtkOrientation orientation)
-{
-	gtk_label_set_angle(m_button_label, (orientation == GTK_ORIENTATION_VERTICAL) ? 270: 0);
-	update_size();
-}
-#endif
 
 //-----------------------------------------------------------------------------
 
@@ -407,12 +425,8 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 {
 	GtkOrientation panel_orientation = xfce_panel_plugin_get_orientation(m_plugin);
 	GtkOrientation orientation = panel_orientation;
-#if (LIBXFCE4PANEL_CHECK_VERSION(4,9,0))
 	gint row_size = size / xfce_panel_plugin_get_nrows(m_plugin);
 	XfcePanelPluginMode mode = xfce_panel_plugin_get_mode(m_plugin);
-#else
-	gint row_size = size;
-#endif
 
 	// Make icon expand to fill button if title is not visible
 	gtk_box_set_child_packing(GTK_BOX(m_button_box), GTK_WIDGET(m_button_icon),
@@ -421,28 +435,26 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 			0, GTK_PACK_START);
 
 	// Load icon
-	GtkStyle* style = gtk_widget_get_style(m_button);
-	gint border = (2 * std::max(style->xthickness, style->ythickness)) + 2;
+	GtkBorder border, padding;
+	GtkStyleContext* context = gtk_widget_get_style_context(m_button);
+	GtkStateFlags state = gtk_style_context_get_state(context);
+	gtk_style_context_get_border(context, state, &border);
+	gtk_style_context_get_padding(context, state, &padding);
+	gint margin_x = border.left + border.right + padding.left + padding.right;
+	gint margin_y = border.top + border.bottom + padding.top + padding.bottom;
 
 	GdkScreen* screen = gtk_widget_get_screen(GTK_WIDGET(m_plugin));
 	GtkIconTheme* icon_theme = G_LIKELY(screen != NULL) ? gtk_icon_theme_get_for_screen(screen) : NULL;
 
-#if (LIBXFCE4PANEL_CHECK_VERSION(4,9,0))
 	gint icon_width_max = (mode == XFCE_PANEL_PLUGIN_MODE_HORIZONTAL) ?
-			6 * row_size - border :
-			size - border;
-	gint icon_height_max = row_size - border;
+			6 * row_size - margin_x :
+			size - margin_x;
+	gint icon_height_max = row_size - margin_y;
 	GdkPixbuf* icon = xfce_panel_pixbuf_from_source_at_size(
 			wm_settings->button_icon_name.c_str(),
 			icon_theme,
 			icon_width_max,
 			icon_height_max);
-#else
-	GdkPixbuf* icon = xfce_panel_pixbuf_from_source(
-			wm_settings->button_icon_name.c_str(),
-			icon_theme,
-			row_size - border);
-#endif
 	gint icon_width = 0;
 	if (G_LIKELY(icon != NULL))
 	{
@@ -451,18 +463,17 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 		g_object_unref(G_OBJECT(icon));
 	}
 
-#if (LIBXFCE4PANEL_CHECK_VERSION(4,9,0))
 	if (wm_settings->button_title_visible || !wm_settings->button_single_row)
 	{
 		xfce_panel_plugin_set_small(m_plugin, false);
 
 		// Put title next to icon if panel is wide enough
 		GtkRequisition label_size;
-		gtk_widget_size_request(GTK_WIDGET(m_button_label), &label_size);
+		gtk_widget_get_preferred_size(GTK_WIDGET(m_button_label), NULL, &label_size);
 		if (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR &&
 				wm_settings->button_title_visible &&
 				wm_settings->button_icon_visible &&
-				label_size.width <= (size - border - icon_width))
+				label_size.width <= (size - margin_x - icon_width))
 		{
 			orientation = GTK_ORIENTATION_HORIZONTAL;
 		}
@@ -471,7 +482,6 @@ gboolean Plugin::size_changed(XfcePanelPlugin*, gint size)
 	{
 		xfce_panel_plugin_set_small(m_plugin, true);
 	}
-#endif
 
 	// Fix alignment in deskbar mode
 	if ((panel_orientation == GTK_ORIENTATION_VERTICAL) && (orientation == GTK_ORIENTATION_HORIZONTAL))
